

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   SmartSurfer Loader 1.0                                        *
    *   Copyright  2004 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-07-05   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



/*

	libs: MSVCRT.LIB kernel32.lib user32.lib 
	opts: /nodefaultlib:"LIBCMT.LIB" /nodefaultlib

*/

#include <windows.h>
#include <tchar.h>

#include "SmartSurfer Hook.h"
#include "devil_compile.h"


typedef FACE * ( * FUNC_FACE )( void );


#define LEVEL_MUTEX		0
#define LEVEL_LAUNCH	1
#define LEVEL_DLL		2
#define LEVEL_HOOK		3


FUNC_FACE get_face = NULL;
FACE * g_FACE = NULL;

HINSTANCE g_hDLL = NULL;
HANDLE hMutex = NULL;

PROCESS_INFORMATION processinformation;



int cleanup( UINT level, bool bDone )
{
	bool bRunning = true;

	if( !bDone )
	{
		level--;
	}

	switch( level )
	{
		case LEVEL_HOOK:
		{
			bRunning = false;

			// Try to remove hook
			if( !g_FACE->stop() )
			{
				MessageBox( NULL, TEXT( "CBT-Hook could not be removed." ), TEXT( "ERROR" ), MB_ICONERROR );
			}
		}
		case LEVEL_DLL:
		{
			// Unload hook dll
			FreeLibrary( g_hDLL );
		}
		case LEVEL_LAUNCH:
		{
			// Kill Smart Surfer if still running
			if( bRunning )
			{
				TerminateProcess( processinformation.hProcess, 0 );
			}
		}
		case LEVEL_MUTEX:
		{
			// Release mutex
			ReleaseMutex( hMutex );
		}
	}

	return 0;
}


int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPTSTR, int )
{
	// Only one running instance
	hMutex = CreateMutex( NULL, TRUE, "SMARTSURFER_LOADER" );
	if( GetLastError() == ERROR_ALREADY_EXISTS )
	{
		// Existing instance found
		return cleanup( LEVEL_MUTEX, false );
	}
	

	STARTUPINFO startupinfo;
	startupinfo.cb				= sizeof( STARTUPINFO );
	startupinfo.lpReserved		= NULL;
	startupinfo.lpDesktop		= NULL;
	startupinfo.lpTitle			= NULL;
	startupinfo.dwFlags			= 0;
	startupinfo.cbReserved2		= 0;
	startupinfo.lpReserved2		= NULL;

	DWORD dwExitCode;


	// Execute SmartSurfer and suspend
	if(	!CreateProcess(
		NULL,						// lpApplicationName
		TEXT( "SmartSurfer.exe" ),	// lpCommandLine
		NULL,						// lpProcessAttributes
		NULL,						// lpThreadAttributes
		FALSE,						// bInheritHandles
		NORMAL_PRIORITY_CLASS | CREATE_SUSPENDED, // dwCreationFlags
		NULL,						// lpEnvironment
		NULL,						// lpCurrentDirectory
		&startupinfo,				// lpStartupInfo
		&processinformation )		// lpProcessInformation
	)
	{
		MessageBox( NULL, TEXT( "\"SmartSurfer.exe\" could not be executed." ), TEXT( "ERROR" ), MB_ICONERROR );
		return cleanup( LEVEL_LAUNCH, false );
	}


	// Load hook dll
	g_hDLL = LoadLibrary( TEXT( "SmartSurfer Hook.dll" ) );
	if( g_hDLL == NULL )
	{
		MessageBox( NULL, TEXT( "\"SmartSurfer Hook.dll\" could not be loaded." ), TEXT( "ERROR" ), MB_ICONERROR );
		return cleanup( LEVEL_DLL, false );
	}


	// Find interface function
	get_face = ( FUNC_FACE )GetProcAddress( g_hDLL, "get_face" );	// ANSI !!!
	if( get_face == NULL )
	{
		return cleanup( LEVEL_DLL, true );
	}
	
	g_FACE = get_face();


	// Install hook
	if( !g_FACE->start( g_hDLL, processinformation.dwProcessId ) )
	{
		MessageBox( NULL, TEXT( "CBT-Hook could not be installed." ), TEXT( "ERROR" ), MB_ICONERROR );
		return cleanup( LEVEL_HOOK, false );
	}


	// Resume SmartSurfer
	ResumeThread( processinformation.hThread );


	// Set us idle
	SetPriorityClass( GetCurrentProcess(), IDLE_PRIORITY_CLASS );


	// Wait for SmartSurfer exit
	while( true )
	{
		GetExitCodeProcess( processinformation.hProcess, &dwExitCode );
		if( dwExitCode != STILL_ACTIVE )
		{
			break;
		}
		Sleep( 2000 );
	}

	return cleanup( LEVEL_HOOK, true );
}